package imagemed.layout.validator;

import imagemed.layout.LayoutPart;
import imagemed.types.enumeration.IMTypeos;

import java.util.*;

public class DefaultschPatientValidator extends DBValidator {

	@Override
	protected void preValidate() {
		//Name, Patient ID and SSN must be filled (but ID is auto-generated)
		setMandatoryError( "0NAM" );
		//		setMandatoryError("0UPI");
		//removed (Andrew)		setMandatoryError("0SSN");
		//Community only		setMandatoryWarning("0PHO");

		//Refresh - Set fields
		String refreshingType = getRefreshType();

		if ( REFRESHTYPE_BUTTON.equals( refreshingType ) ) {
			if ( "CopyGuarantor".equals( getRefreshingMacroID() ) ) { //Guarantor copy - from Patient to PatientInsurer
				RecordIterator copyTo = getRefreshingRecords();
				while ( copyTo.next() ) { //there will be 1
					String[][] colids = { { "0SSN", "0BAS" }, { "0BDT", "0BAB" }, { "0PAD", "0BAE" },
							{ "0PA2", "0BA2" }, { "0PCI", "0BCI" }, { "0PSE", "0BST" }, { "0PZI", "0BZI" }, { "0PHO", "0BPH" } };
					copyValues( colids, null, copyTo );
				}
			}
			else if ( "CopyGuardian".equals( getRefreshingMacroID() ) ) { //Guardian copy -  from Patient to Patient
				String[][] colids = { { "0PAD", "0GUA" }, { "0PA2", "0PG2" },
						{ "0PCI", "0PGC" }, { "0PSE", "0PGS" }, { "0PZI", "0PGZ" }, { "0PHO", "0GUP" } };
				copyValues( colids, null, null );
			}
			else if ( "CopyEmergency".equals( getRefreshingMacroID() ) ) { //Emergency contact copy -  from Patient to Patient
				String[][] colids = { { "0PHO", "0PEH" } };
				copyValues( colids, null, null );
			}
		}
		else if ( REFRESHTYPE_FIELDCHANGE.equals( refreshingType ) ) {
			String refreshingColid = getRefreshingColid();
			//DoB -> Minor
			if ( "0BDT".equals( refreshingColid ) ) {
				String birthDate = getValue( "0BDT" );
				if ( !"".equals( birthDate ) ) {
					try {
						GregorianCalendar patCal = getCalendarFromDate( birthDate );
						patCal.add( GregorianCalendar.YEAR, 18 );
						GregorianCalendar nowCal = new GregorianCalendar();

						//birth + 18 < now
						if ( patCal.before( nowCal ) ) {
							setValue( "0MIN", "no" );
						}
						else {
							setValue( "0MIN", "yes" );
						}
					}
					catch ( Exception e ) {
					}
				}
			}
		}

		//Minor -> Guardian name mandatory 
		if ( "yes".equals( getValue( "0MIN" ) ) ) {
			setMandatoryWarning( "0GUN" );
		}

	}


	@Override
	public void validate() {

		//		RecordIterator patientInsurer = getIterator( m_schPatientInsurerTable );
		//		if( patientInsurer.isEmpty() ) {
		//			m_warnings.add("A patient should have at least one insurer defined.");
		//		}

		//TR-IN-1.1If patient is a minor, confirm guardian is assigned 
		//TODO: and parental consent form is recorded.
		//		if( "yes".equals( getValue( "0MIN" ) ) ) {
		//			mustBeFilled( new String[]{"0GUN"}, null, "Patient is a minor, so the following must be done:" );
		//		}

		//SSN, ID, Name collision detection
		//
		shouldBeUnique( "0SSN", m_patientPage, new String[]{ "0NAM", "0UPI" } );

		shouldBeUnique( "0UPI", m_patientPage, new String[]{ "0NAM", "0SSN" } );

		shouldBeUnique( "0NAM", m_patientPage, new String[]{ "0UPI", "0SSN" } );


		//Primary fields
		setPrimaryWarning( new String[]{ "0NAM", "0SSN", "0BDT" } );

		//Check-in detection: if there's an order of this patient in at least Checked in state
		//OR layout-subtype is CheckIn 

		Properties filters = new Properties();
		String insidePatientID = getValue( "0PAI" );
		if ( !"".equals( insidePatientID ) ) {
			filters.setProperty( "0ZPI", insidePatientID );
			filters.setProperty( "0ZST", IMTypeos.STATE_CHECKEDIN + ":" );
			DBIterator allOrders = getDBIterator( new String[]{ "0ZAN" }, filters );
			if ( LayoutPart.SCHPATIENTCHECKINLAYOUT.equals( getFullLayoutType() ) ||
					( !allOrders.isEmpty() ) ) {
				//Check-in stuff
				shouldBeFilled( new String[]{ "0SEX" } );
			}
		}
	}

}
